@@ -33,13 +33,11 @@ from tenacity import Retrying, stop_after_attempt, wait_fixed, wait_random
 from airflow.compat.functools import cached_property
 from airflow.exceptions import AirflowException
 from airflow.hooks.base import BaseHook
-
-try:
-    from airflow.utils.platform import getuser
-except ImportError:
-    from getpass import getuser  # type: ignore[misc]
+from airflow.utils.platform import getuser
+from airflow.utils.types import NOTSET, ArgNotSet
 
 TIMEOUT_DEFAULT = 10
+CMD_TIMEOUT = 10
 
 
 class SSHHook(BaseHook):
@@ -62,6 +60,9 @@ class SSHHook(BaseHook):
         predefined in the connection of `ssh_conn_id`.
     :param timeout: (Deprecated). timeout for the attempt to connect to the remote_host.
         Use conn_timeout instead.
+    :param cmd_timeout: timeout (in seconds) for executing the command. The default is 10 seconds.
+        Nullable, `None` means no timeout. If provided, it will replace the `cmd_timeout`
+        which was predefined in the connection of `ssh_conn_id`.
     :param keepalive_interval: send a keepalive packet to remote host every
         keepalive_interval seconds
     :param banner_timeout: timeout to wait for banner from the server in seconds
@@ -111,6 +112,7 @@ class SSHHook(BaseHook):
         port: int | None = None,
         timeout: int | None = None,
         conn_timeout: int | None = None,
+        cmd_timeout: int | ArgNotSet | None = NOTSET,
         keepalive_interval: int = 30,
         banner_timeout: float = 30.0,
         disabled_algorithms: dict | None = None,
@@ -126,6 +128,7 @@ class SSHHook(BaseHook):
         self.port = port
         self.timeout = timeout
         self.conn_timeout = conn_timeout
+        self.cmd_timeout = cmd_timeout
         self.keepalive_interval = keepalive_interval
         self.banner_timeout = banner_timeout
         self.disabled_algorithms = disabled_algorithms
@@ -177,6 +180,12 @@ class SSHHook(BaseHook):
                 if "conn_timeout" in extra_options and self.conn_timeout is None:
                     self.conn_timeout = int(extra_options["conn_timeout"])
 
+                if "cmd_timeout" in extra_options and self.cmd_timeout is NOTSET:
+                    if extra_options["cmd_timeout"]:
+                        self.cmd_timeout = int(extra_options["cmd_timeout"])
+                    else:
+                        self.cmd_timeout = None
+
                 if "compress" in extra_options and str(extra_options["compress"]).lower() == "false":
                     self.compress = False
 
@@ -230,6 +239,9 @@ class SSHHook(BaseHook):
         if self.conn_timeout is None:
             self.conn_timeout = self.timeout if self.timeout else TIMEOUT_DEFAULT
 
+        if self.cmd_timeout is NOTSET:
+            self.cmd_timeout = CMD_TIMEOUT
+
         if self.pkey and self.key_file:
             raise AirflowException(
                 "Params key_file and private_key both provided.  Must provide no more than one."
@@ -290,17 +302,16 @@ class SSHHook(BaseHook):
             known_hosts = os.path.expanduser("~/.ssh/known_hosts")
             if not self.allow_host_key_change and os.path.isfile(known_hosts):
                 client.load_host_keys(known_hosts)
-        else:
-            if self.host_key is not None:
-                client_host_keys = client.get_host_keys()
-                if self.port == SSH_PORT:
-                    client_host_keys.add(self.remote_host, self.host_key.get_name(), self.host_key)
-                else:
-                    client_host_keys.add(
-                        f"[{self.remote_host}]:{self.port}", self.host_key.get_name(), self.host_key
-                    )
+
+        elif self.host_key is not None:
+            # Get host key from connection extra if it not set or None then we fallback to system host keys
+            client_host_keys = client.get_host_keys()
+            if self.port == SSH_PORT:
+                client_host_keys.add(self.remote_host, self.host_key.get_name(), self.host_key)
             else:
-                pass  # will fallback to system host keys if none explicitly specified in conn extra
+                client_host_keys.add(
+                    f"[{self.remote_host}]:{self.port}", self.host_key.get_name(), self.host_key
+                )
 
         connect_kwargs: dict[str, Any] = dict(
             hostname=self.remote_host,
@@ -460,10 +471,13 @@ class SSHHook(BaseHook):
         command: str,
         get_pty: bool,
         environment: dict | None,
-        timeout: int | None,
+        timeout: int | None = None,
     ) -> tuple[int, bytes, bytes]:
         self.log.info("Running command: %s", command)
 
+        if timeout is None:
+            timeout = self.cmd_timeout  # type: ignore[assignment]
+
         # set timeout taken as params
         stdin, stdout, stderr = ssh_client.exec_command(
             command=command,
@@ -492,7 +506,8 @@ class SSHHook(BaseHook):
         # read from both stdout and stderr
         while not channel.closed or channel.recv_ready() or channel.recv_stderr_ready():
             readq, _, _ = select([channel], [], [], timeout)
-            timedout = len(readq) == 0
+            if timeout is not None:
+                timedout = len(readq) == 0
             for recv in readq:
                 if recv.recv_ready():
                     output = stdout.channel.recv(len(recv.in_buffer))
@@ -528,3 +543,12 @@ class SSHHook(BaseHook):
         exit_status = stdout.channel.recv_exit_status()
 
         return exit_status, agg_stdout, agg_stderr
+
+    def test_connection(self) -> tuple[bool, str]:
+        """Test the ssh connection by execute remote bash commands"""
+        try:
+            with self.get_conn() as conn:
+                conn.exec_command("pwd")
+            return True, "Connection successfully tested"
+        except Exception as e:
+            return False, str(e)
